/* Example of stack and heap allocated memory using C strings
 * Also shows where code and literals are allocated
 * CS 208 w20, Aaron Bauer
 *
 * compile with "gcc -Og -g -o str_test str_test.c"
 * -Og sets a compiler optimization level conducive to debugging
 * -g includes extra debugging information in the executable
 * -o str_test sets the name of the output file (i.e., compiled executable) to str_test
 * the default gcc output executable name is a.out
 * Note how the addresses of stack allocated > heap > literals > code
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int main() {
  printf("address of the function main (code): %p\n", &main);

  char *str_arr[3] = {"hello", "there", "friend"};
  printf("address of str_arr (stack): %p\n", &str_arr);
  printf("address of the first element of str_arr (stack): %p\n", &str_arr[0]); // same as address of the array
  printf("address of the second element of str_arr (stack): %p\n", &str_arr[1]);
  // array elements layed out sequentially, aligned to 8-byte chunks
  
  char *s = "hello world";
  printf("address of s (stack): %p\n", &s); // stack grows down
  printf("address of the literal \"hello world\": %p\n", s);

  char *str_arr2[3];
  str_arr2[0] = (char *) malloc(100 * sizeof(char)); // dynamically allocate 100 bytes on the heap
  printf("pointer value stored at str_arr2[0] (heap): %p\n", str_arr2[0]);
  // see http://www.cplusplus.com/reference/cstring/strcpy/
  strcpy(str_arr2[0], "on the heap"); // without malloc this line probably causes a segmentation fault
  printf("address of str_arr2: %p\n", &str_arr2); // the pointers in the array still stack allocated  
}
